// 
// Copyright (C) 2008 Christian Pointner, mur.at
//                    <equinox@spreadspace.org>
//
// This is free software, licensed under the GNU General Public License v2.
// See /LICENSE for more information.
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ncurses.h>
#include <stdarg.h>

#include "../mur-lcm.h"

WINDOW *lcd;
#define LCD_COLS 16
#define LCD_ROWS 2

void init_lcm()
{
  initscr();
	cbreak();
	noecho();
  curs_set(0);

  lcd = newwin(LCD_ROWS+2, LCD_COLS+2, (LINES - LCD_ROWS+2) / 2, (COLS - LCD_COLS+2) / 2);
  box(lcd, 0 , 0);
	keypad(lcd, TRUE);
  wrefresh(lcd);
}

void destroy_lcm()
{
  delwin(lcd);
  endwin();
}

void clear_lcd()
{
  if(!lcd)
    return;
  
  wclear(lcd);
  box(lcd, 0 , 0);
  wrefresh(lcd);
}

void print_lcd(int row, int col, const char* fmt, ...)
{
  va_list args;

  if(row >= LCD_ROWS || col >= LCD_COLS)
    return;

  char buf[LCD_COLS+1];
  va_start(args, fmt);
  vsnprintf(buf, (LCD_COLS+1 - col), fmt, args);
  va_end(args);

  mvwaddstr(lcd, row+1, col+1, buf);
  wrefresh(lcd);
}

int read_button()
{
  if(!lcd)
    return 0;

  int ch = wgetch(lcd);
  switch(ch)
  {
  case KEY_LEFT: return BTN_ESC;
  case KEY_RIGHT: return BTN_ENTER;
  case KEY_UP: return BTN_UPPER;
  case KEY_DOWN: return BTN_DOWN;
  } 
  return 0;
}
