// 
// Copyright (C) 2008 Christian Pointner, mur.at
//                    <equinox@spreadspace.org>
//
// This is free software, licensed under the GNU General Public License v2.
// See /LICENSE for more information.
//

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "mur-lcm.h"
#include "menu.h"

extern FILE *yyin;
extern void yyinit(menu_entry* first);
extern int yyparse(void);
char* config_file = NULL;

void printUsage()
{
  fprintf(stderr, "mur-lcm 0.1\n");
  fprintf(stderr, "  written by Christian Pointner <equinox@spreadspace.org>\n\n");

  fprintf(stderr, "Usage: mur-lcm <config-file> [<hostname>]\n");
}

void init()
{
  init_lcm();

#ifdef TARGET_NATIVE
/* // the native binary should run in background */
/*   pid_t pid; */

/*   pid = fork(); */
/*   if(pid) exit(0);   */
/*   setsid(); */
/*   pid = fork(); */
/*   if(pid) exit(0); */
  
/*   int fd; */
/* //  for (fd=getdtablesize();fd>=0;--fd) // close all file descriptors */
/*   for (fd=0;fd<=2;fd++) // close std file descriptors */
/*     close(fd); */
/*   fd=open("/dev/null",O_RDWR);        // stdin */
/*   dup(fd);                            // stdout */
/*   dup(fd);                            // stderr */
/*   umask(027);  */
#endif
}

void cleanup(menu_entry** menu_nil)
{
  clearMenu(menu_nil);
  destroy_lcm();
}

int catched_sig = _NSIG;

void sigUsrHandler(int sig)
{
  catched_sig = sig;
}

void intro(char* hostname)
{
  clear_lcd();
  print_lcd(0, 0, "   m u r . a t  ");
  print_lcd(1, 0, "mur-lcm 0.1");
  sleep(2);
  clear_lcd();

  print_lcd(0, 0, "xorp@mur: %s", hostname);
  print_lcd(1, 0, "booting ");

  int i = 0;
  while(1) {
    if(catched_sig == SIGUSR1)
      break;

    sleep(1);

    switch(i) {
    case 0: print_lcd(1,8, "   "); break;
    case 1: print_lcd(1,8, ".  "); break;
    case 2: print_lcd(1,8, ".. "); break;
    case 3: print_lcd(1,8, "..."); break;
    }
    i++;
    if(i>3)
      i = 0;
  }

  print_lcd(1, 0, "finished ...");
  sleep(2);
}

void test()
{
  clear_lcd();
  print_lcd(0, 0, "hello world!");

  int btn = 0;
  while(1)
  {
    btn = read_button();
    print_lcd(1, 0, "0x%02X", btn);
  }  
}

void printNextMenuEntries(menu_entry* entry)
{
  clear_lcd();
  menu_entry* current;
  int i = 0;
  for(current = entry; current && i < 2; ++i, current = current->next) {
    print_lcd(i,0,"%s",current->name);
    if(current->type == SUBMENU)
      print_lcd(i,15,">");
    if(current->type == COMMAND)
      print_lcd(i,15,"*");
    if(current->type == SHOW)
      print_lcd(i,15,"|");

  }
}

void executeCommands(const char* name, cmdlist_entry* list)
{
  clear_lcd();
  print_lcd(0,0,"x: %s", name);
  if(!list)
    print_lcd(1,0,"<empy list>");
  else {
    cmdlist_entry* current;
    int ret = 0;
    for(current = list; current; current = current->next)
      ret = system(current->command);
    
    print_lcd(1,0,"returned %d", ret);
  }
  sleep(1);
}

void executeShow(const char* name, const char* cmd)
{
  clear_lcd();
  print_lcd(0,0,"s: %s", name);
  if(!cmd)
    print_lcd(1,0,"<no command>");
  else {
    FILE* p = popen(cmd, "r");
    if(p) {
      char buf[17];
      if(fgets(buf, 17, p)) {
        char* tmp = strchr(buf, '\n');
        if(tmp)
          tmp[0] = 0;
        print_lcd(1,0,"%s", buf);
      }
      pclose(p);
    }
    else
      print_lcd(1,0, "execute error");
  }
  sleep(1);
}

void menu(menu_entry* menu_root)
{
  menu_entry* current = getSubEntry(menu_root);
  if(!current) {
    print_lcd(0,0,"<menu empty>");
    print_lcd(1,0,config_file);
    return;
  }

  clear_lcd();
  int btn = 0;
  while(1)
  {
    printNextMenuEntries(current);
    btn = read_button();
    switch(btn)
    {
    case BTN_UPPER: {
      if(current->prev) 
        current = current->prev; 
      break;
    }
    case BTN_DOWN: {
      if(current->next)
        current = current->next; 
      break;
    }
    case BTN_ENTER: {
      if(current->type == SUBMENU) {
        if(current->sub_entry)
          current = current->sub_entry;
      }
      else if(current->type == COMMAND) {
        executeCommands(current->name, current->cmd_list);
      }
      else if(current->type == SHOW) {
        executeShow(current->name, current->show_cmd);
      }
      break;
    }
    case BTN_ESC: {
        menu_entry* ptr = getSuperEntry(current); 
        if(ptr && ptr->type != NIL)
          current = ptr; 
        break;
      }
    }
  }  
}

int main(int argc, char** argv)
{
  if(argc < 2 || argc > 3) {
    printUsage();
    return(-1);
  }

  yyin = fopen(argv[1], "r");
  if(!yyin) { 
    fprintf(stderr, "Error: can't open file for reading: %s", argv[1]);
    return(-1);
  }

  menu_entry* menu_root = newMenuEntry();
  menu_root->type = NIL;
  config_file = argv[1];
  yyinit(menu_root);
  yyparse();

  printMenu(menu_root, 0);
 
  init();

  (void) signal(SIGUSR1, sigUsrHandler);
  (void) signal(SIGUSR2, sigUsrHandler);

  if(argc > 2)
    intro(argv[2]);
  else
    intro("unknown");

/*   test(); */
  menu(menu_root);

  cleanup(&menu_root);
  return 0;
}
