// 
// Copyright (C) 2008 Christian Pointner, mur.at
//                    <equinox@spreadspace.org>
//
// This is free software, licensed under the GNU General Public License v2.
// See /LICENSE for more information.
//

#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/io.h>

#include "../mur-lcm.h"

#define LCD_COLS 16
#define LCD_ROWS 2

#define 	LPT_DATA_OUT		0x378		//Which Paralle Port use 
#define		LPT_STATUS_IN		0x379		//PORT+1
#define 	LPT_CTL_OUT		0x37a		//PORT+2
/* All refference the LCM Data Sheet */
#define 	LCD_CLEAR		0x001
#define 	LCD_HOME		0x002
#define 	LCD_ENT			0x004
#define 	LCD_ENT_INC		0x002
#define 	LCD_ENT_SHIFT		0x001
#define 	LCD_DISP		0x008
#define 	LCD_DISP_ON		0x004
#define 	LCD_DISP_CURS_ON	0x002
#define 	LCD_DISP_BLINK_ON	0x001
#define 	LCD_SET_FUNC		0x020
#define 	LCD_FUNC_8BITS		0x010
#define 	LCD_FUNC_2LINE		0x008
#define 	LCD_SET_CG_ADDR		0x040
#define 	LCD_SET_DD_ADDR		0x080
#define 	LCD_KEY_DOWN		0x0ef
#define 	LCD_KEY_UP		0x0cf
#define 	LCD_KEY_RIGHT		0x0e7
#define 	LCD_KEY_LEFT		0x0c7

void	lcd_cmd(int	para);
void	lcd_goto_rc(int	row,int col);
void	lcd_write(int	wdata);

void init_lcm()
{
	iopl(3);	/*allow to access all 65536 IO ports*/

	// set LCD interface for 8-bits, and 2-line display
	lcd_cmd(LCD_SET_FUNC | LCD_FUNC_8BITS | LCD_FUNC_2LINE);

	// turn on the backlight, cursor is off, blink is off
	lcd_cmd(LCD_DISP | LCD_DISP_ON);

	// set entry mode/cursor move direction to 'increase' (right)
	// display will not be shifted (ie: insert mode off)
	lcd_cmd(LCD_ENT | LCD_ENT_INC);

	// set CG address to 8 - char(1) -- we will skip char(0), since
	// it may be problematic for NULL-term strings (ie:lcd_print_string)
	lcd_cmd(LCD_SET_CG_ADDR | 8);

  lcd_cmd(LCD_HOME);
  usleep(10);

  lcd_cmd(LCD_CLEAR);
  usleep(10);
	return;
}

void destroy_lcm()
{
}

void clear_lcd()
{
  lcd_cmd(LCD_CLEAR);
  usleep(10);
}

void print_lcd(int row, int col, const char* fmt, ...)
{
  va_list args;

  if(row >= LCD_ROWS || col >= LCD_COLS)
    return;

  char buf[LCD_COLS+1];
  va_start(args, fmt);
  int len = vsnprintf(buf, (LCD_COLS+1 - col), fmt, args);
  va_end(args);
  if(len <= 0)
    return;

  lcd_goto_rc(row,col);
  int i;
  for(i=0; i<len; i++)
    lcd_write(buf[i]);
}


int read_button()
{
	int stat1, stat2;

  while(1) {
    stat1 = inb(LPT_STATUS_IN);
    usleep(5000);
    stat2 = inb(LPT_STATUS_IN);
    
    if (stat1 == stat2)
      continue;
    
    switch(stat2) {
    case LCD_KEY_UP: return BTN_UPPER;
    case LCD_KEY_DOWN: return BTN_DOWN;
    case LCD_KEY_RIGHT: return BTN_ENTER;
    case LCD_KEY_LEFT: return BTN_ESC;
    default: return 0;
    }
  }
}



/* this is form linuxlcm.c which can be found on the Lanner Product CD */

/*	## lcd_cmd
    ## INPUT: send byte as a command.
    ## This routine will set the LCD lines RS==low, R/W==low and
    ## then write the byte in %al to the lines DB0-DB7 -- all this
    ## good stuff is connected to the paralel port, DB0-DB7 is
    ## accessed through the DATA PORT and RS, R/W are accessed through
    ## the CONTROL PORT of the parallel port.				*/
void lcd_cmd(int para)
{
	outb(0x0c8,LPT_CTL_OUT);
  outb(para,LPT_DATA_OUT);
  outb(0x0ca,LPT_CTL_OUT);
  usleep(10);
	return;
}

void lcd_goto_rc(int row,int col)
{
	col&=0x01f;
	col|=LCD_SET_DD_ADDR;
	if (row)
		col|=0x040;
	lcd_cmd(col);
	return;
}

/*	## lcd_write
    ## INPUT: byte to be written to DISPLAY or CG RAM in LCD.
    ## This routine will set the LCD line RS==high, R/W==low and
    ## then send the data in %al to DB0-DB7 of the LCD.
    ## If the LCD is expecting Display data (default mode), the contents
    ## will appear on the LCD screen (so long as the cursor location
    ## is valid for writing -- there are off-screen locations!!!)
    ## If the LCD is expecting CG data, the contents will go into
    ## the CG RAM, etc...							*/
void lcd_write(int wdata)
{
	outb(0x0c0,LPT_CTL_OUT);
	outb(wdata,LPT_DATA_OUT);
	outb(0x0c2,LPT_CTL_OUT);
	usleep(10);
	return;
}
