%{
// 
// Copyright (C) 2008 Christian Pointner, mur.at
//                    <equinox@spreadspace.org>
//
// This is free software, licensed under the GNU General Public License v2.
// See /LICENSE for more information.
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "menu.h"

void yyerror(const char *);
int yylex(void);

int line_cnt = 1;
extern char* config_file;
menu_entry* current = NULL;

void yyinit(menu_entry* first)
{
  current = first;
}

char* stradd(char *dest, const char *src)
{
  int len = strlen(dest);
  len += strlen(src);
  char* tmp = realloc(dest, len+2);
  if(!tmp)
    exit(2);

  strcat(tmp, " ");
  strcat(tmp, src);
  return tmp;
}
%}

%union 
{
  char null;
  char *string;
  cmdlist_entry* cmd_list;
  menu_entry* menu;
} 

%token <string> TOK_MENU;
%token <string> TOK_COMMAND;
%token <string> TOK_SHOW;
%token <string> TOK_DISPLAY;
%token <string> TOK_NAME;
%token <string> TOK_WORD;
%token <null> TOK_OPEN;
%token <null> TOK_CLOSE;
%token <null> TOK_SEMICOLON;
%token <null> TOK_QUOTE;

%type <string> strings
%type <string> string
%type <cmd_list> commands
%type <string> command
%type <menu> cmdblock_head
%type <null> cmdblock;
%type <null> showblock;
%type <null> displayblock;
%%

menu: 
               | menu menuentry
               ;

menuentry:       submenu
               | cmdblock
               | showblock
               | displayblock
               ;

submenu:         submenu_head TOK_OPEN menu TOK_CLOSE TOK_SEMICOLON
{
  current = getSuperEntry(current);
}
;

submenu_head:    TOK_MENU TOK_NAME
{
  free($1);
  current = addSubNamedEntry(current, $2, SUBMENU);
  free($2);
}
;

cmdblock:        cmdblock_head TOK_OPEN commands TOK_CLOSE TOK_SEMICOLON
{
  $1->cmd_list = $3;
}
;

cmdblock_head:   TOK_COMMAND TOK_NAME
{
  free($1);
  $$ = addSubCommandList(current, $2, NULL);
  free($2);
}
;

commands:        command { $$ = NULL; addCommand(&($$), $1); free($1); }
               | commands TOK_SEMICOLON command
{
  addCommand(&($$), $3);
  free($3);
}
;

command:         TOK_QUOTE strings TOK_QUOTE
{
  $$ = $2;
}
;

showblock:        TOK_SHOW TOK_NAME TOK_OPEN command TOK_CLOSE TOK_SEMICOLON
{
  free($1);
  addSubShowCommand(current, $2, $4);
  free($2);
  free($4);
}
;

displayblock:        TOK_DISPLAY TOK_OPEN command TOK_CLOSE TOK_SEMICOLON
{
  free($1);
  addSubNamedEntry(current, $3, DISPLAY);
  free($3);
}
;

strings:         string
               | strings string
{
  $$ = stradd($$, $2);
  free($2);
}
;

string:          TOK_NAME
               | TOK_WORD
               | TOK_MENU;
               | TOK_COMMAND;
               | TOK_SHOW;
               | TOK_DISPLAY;
               ;

%%

void yyerror (const char *string)
{
  fprintf(stderr, "%s:%d %s\n", config_file, line_cnt, string);
  exit(1);
}
    
